/*
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the Ark Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <Ark/ArkTypes.h>

#include <stdlib.h>
#include <assert.h>
#include <string.h>

static inline int max (int x, int y)
{ return x > y ? x : y; }


static inline int min (int x, int y)
{ return x < y ? x : y; }

// Get the length of the string vssprint(format, args) will write.
// This is used to ensure the buffer given to vssprintf is big enough
// and there will be no buffer overflow (it can hang the system, but it
// can also be a security hole). FIXME: I took it somewhere, don't
// remember where. As I'm not sure about the licence, it'll eventually
// need to be rewritten.

int strGetMaxLen (const char *format, va_list args)
{
  int maxlen = 0;

  for (const char *s = format; *s != '\0'; s++)
  {
    int len = 0, width = 0, prec = 0;

    if (*s == '%')
    {
      s ++;

      if (*s == '%')
        len = 1;

      /* ===== Get width ===== */
      for (; *s != '\0'; s++)
      {
        if (*s == '#')
          len += 2;   // for '0x'
        if (*s == '*')
          width = va_arg (args, int);
        else if (*s == '-'||*s == '+'||*s == '0'||*s == ' ')
          ;
        else // hit non-flag character
          break;
      }

      if (width == 0)
      {
        width = atoi (s);
        for (; isdigit (*s) && *s != '\0'; s++)
          ;
      }

      assert (width >= 0);

      /* ===== Get Precision ===== */
      if (*s == '.')
      {
        s++;
        if (*s == '*')
          prec = va_arg (args, int);
        else
        {
          prec = atoi (s);
          for (; isdigit (*s) && *s != '\0'; s++)
            ;
        }
      }

      if (*s == 'F' || *s == 'N' || *s == 'L')
        s++;

      /* ===== Now get the size for string & chars ===== */
      switch (*s)
      {
        // single characters
        case 'c':
        case 'C':
          len = 2;
          (void) va_arg (args, int);
          break;

        // strings
        case 's':
	  {
            /* If the string is NULL, "(null)" will be appended */
            const char *s2 = va_arg (args, char *);
            int slen = s2 ? strlen (s2) : 6;

            len = slen > 1 ? slen : 1;
          }
          break;
      }

      /* ===== Fix the len for strings ===== */
      if (len)
      {
        len = max (len, width);
        if (prec) len = min (len, prec);
      }
      else
      {
        switch (*s)
        {
          // integers
          case 'd':
          case 'i':
          case 'u':
          case 'x':
          case 'X':
          case 'o':
            (void) va_arg(args, int);
            len = 32;
            len = max(len, width+prec);
            break;

          // floats
          case 'e':
          case 'f':
          case 'g':
          case 'G':
            (void) va_arg(args, double);
            len = 128;
            len = max(len, width+prec);
            break;

          case 'p':
            (void) va_arg(args, void*);
            len = 32;
            len = max(len, width+prec);
            break;

          // no output
          case 'n':
            (void) va_arg(args, int*);
            break;
        }
      }
    }
    else
      len = 2;

    maxlen += len;
  }

  return maxlen;
}

