/*
** ArkRPG - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the ArkRPG Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef CLIENT_LABELWIDGET_H
#define CLIENT_LABELWIDGET_H

#include <Ark/ArkString.h>
#include <Client/Client.h>

namespace Client
{
   class LabelWidget : public Widget
   {
      protected:
	 /** The text to be displayed in the label */
	 Ark::String m_Text;

	 /** The position of the label */
	 int m_X, m_Y;

      public:
	 /**
	  * Constructor with text
	  * \param text The text to be displayed in the label
	  */
	 LabelWidget(const Ark::String& text);

	 /**
	  * Sets the text in the label
	  * \param text The text to be displayed in the label
	  */
	 void SetText(const Ark::String& text);

	 /**
	  * Returns the text displayed by the label
	  * \return The text displayed by the label
	  */
	 const Ark::String& GetText() const;

	 /**
	  * Sets the position of the label
	  * \param x x coordinate of the label
	  * \param y y coordinate of the label
	  */
	 void SetPosition(int x, int y);

	 /**
	  * Gets the x position coordinate of the label
	  * \return The x position coordinate of the label
	  */
	 int GetXPosition() const
	 {
	    return m_X;
	 }

	 /**
	  * Gets the y position coordinate of the label
	  * \return The y position coordinate of the label
	  */
	 int GetYPosition() const
	 {
	    return m_Y;
	 }

	 /**
	  * Displays the label
	  */
	 virtual void Render(UIRenderer *renderer);
   };
}

#endif

