/* $Id: TerrainView.cpp,v 1.34 2002/10/28 22:31:17 mrq Exp $
**
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the ArkRPG Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef WIN32
#include <windows.h>
#endif

#include <sstream>

#include <math.h>

#include <Client/GLClient.h>
#include <Client/TerrainView.h>
#include <Client/MessageView.h>

#if 0
namespace Ark
{
   extern bool g_UpdateVis;
   extern bool g_DrawTerrain;
   extern bool g_DrawEntities;
   extern bool g_WireFrame;
}
#endif

namespace Client
{

   TerrainView::TerrainView (Client *client)
   {
      m_Client = client;
      m_RotYSpeed = 
	 m_RotXSpeed = 
	 m_RotY = 0.0;
      m_RotX = 45.0;
      
      m_Scale = 5.0;
      m_ScaleSpeed = 0.0;
      m_FrameCounter = 0;
      m_FPS = 0;

      m_MessageView = new MessageView();
   }

   TerrainView:: ~TerrainView ()
   {
      delete m_MessageView;
   }
      
   Ark::Vector3 
   UnProject (scalar sX, scalar sY, scalar mz,
	      Ark::Matrix44 invmat)
   {
      Ark::Vector3 R = invmat.Transform(Ark::Vector3 (sX, sY, mz));
      scalar w = (sX * invmat.M(0,3) +
		  sY * invmat.M(1,3) +
		  mz * invmat.M(2,3) +
		  invmat.M(3,3));

      R.Scale (1.0f/w);
      return R;
   }

   Ark::Vector3
   TerrainView::TraceMousePos (const Ark::Vector3 &pov)
   {
      GLClient *client = (GLClient*) m_Client;

      int mx, my;
      SDL_GetMouseState (&mx, &my);

      // Invert y, because of OGL coord system
      my = client->m_Height - my;
      
      if (mx < 0 || my < 0 ||
	  mx > client->m_Width || my > client->m_Height)
	 return Ark::Vector3();
      
      Ark::Matrix44 projmat, modelmat;

      glGetFloatv (GL_PROJECTION_MATRIX, &projmat.M(0,0));
      glGetFloatv (GL_MODELVIEW_MATRIX, &modelmat.M(0,0));

      modelmat.Multiply (projmat);
      modelmat.Invert();

      scalar sX = 2.0f * (scalar(mx) / client->m_Width) - 1.0f;
      scalar sY = 2.0f * (scalar(my) / client->m_Height) - 1.0f;

      Ark::Vector3 lookat = UnProject (sX, sY, 1.0f, modelmat);

      Ark::Vector3 dir = lookat - pov;
      dir.Scale (10000.0f);

      lookat = pov + dir;
      std::vector<Ark::Collision> collisions;
      m_Client->GetWorld()->RayTrace (Ark::Ray(pov, lookat),
				      Ark::Collision::POSITION |
				      Ark::Collision::ENTITY |
				      Ark::Collision::WORLD,
				      collisions);
      if (collisions.size())
      {
	 m_LastHit = collisions[0];
	 return collisions[0].m_Pos;
      }

      return Ark::Vector3 (0.0, 0.0, 0.0);
   }
   
   void
   TerrainView::Render (UIRenderer *ui)
   {
      GLClient *client = (GLClient*) m_Client;

      Ark::Vector3 pointofview(0.0f, 0.0f, -1.0f);      
      Ark::Camera cam;

      scalar dt = m_Timer.GetDelta();
      m_Timer.Update();
      
      m_RotY += m_RotYSpeed * dt;
      m_RotX += m_RotXSpeed * dt;
      m_Scale *= 1.0f + (m_ScaleSpeed * dt);

      /// Clamp scale..
      m_Scale = std::min (16.0f,  std::max (m_Scale, 3.0f));
      
      /// Clamp X rotation
      m_RotX = std::min (89.0f, std::max(m_RotX, 15.0f));

      /// =============================================================
      /// Compute point of view..
      Ark::Matrix44 m;
      m.RotateX (m_RotX);
      pointofview = m.Transform (pointofview);
      m.RotateY (m_RotY);
      pointofview = m.Transform (pointofview);
      pointofview.Scale (m_Scale);
      
      ui->Rdr()->SetViewport (0, 0, client->m_Width, client->m_Height);
      
      if (m_Client->GetPlayer())
      {
	 Ark::BBox bb = m_Client->GetPlayer()->m_MState.ExtractBbox();
	 cam.m_LookAt = m_Client->GetPlayer()->m_MState.m_Position;
	 
	 cam.m_LookAt.Y += (bb.m_Max.Y - bb.m_Min.Y)/2.0f;
      }
      else
	 cam.m_LookAt = Ark::Vector3(128.0f, 5.0f, 128.0f);
      
      cam.m_PointOfView = cam.m_LookAt + pointofview;
      cam.m_FOV = 60.0;

      if (m_Client->GetWorld())
      {
	 Ark::Timer timer;
	 m_Client->GetWorld()->Render (*ui->Rdr(), cam);

	 m_LastMousePos = TraceMousePos (cam.m_PointOfView);

	 // Make "our" model follow the mouse.
	 Ark::Vector3 model_lookat =
	    m_LastMousePos - 
	    m_Client->GetPlayer()->m_MState.m_Position;
	 
	 model_lookat.Y = 0.0;
	 model_lookat.Normalize();

	 //scalar yaw = -(atan2(model_lookat.Z, model_lookat.X) * 180 / PI);
	 scalar mdl_yaw, mdl_pitch, mdl_rot;
	 
	 m_Client->GetPlayer()->m_MState.m_Rotation.GetEuler
	    (&mdl_yaw, &mdl_pitch, &mdl_rot);
	 //m_Client->GetPlayer()->m_MState.SetController (0, yaw-mdl_yaw);
      }
      else
	 m_LastMousePos = Ark::Vector3 (0.0, 0.0, 0.0);

      //-------------- FPS COUNTER
      ui->Rdr()->SetIdentity ();

      Ark::Font *fp = ui->GetFont ("arial1-1-ffffff");

      Ark::String str;
      if (m_FrameCounter == 30)
      {
	 m_FrameCounter = 0;
	 m_FPS = 30.0f / m_FrameTimer.GetDelta();
	 m_FrameTimer.Update();
      }
      else
	 m_FrameCounter++;

      {
	  std::ostringstream os;
	  os << (int)m_FPS << " fps";
	  str = os.str();
      }

      int ypos = client->m_Height - ui->GetStringHeight (fp,str) - 5;
      ui->DrawString (fp, str, 5, ypos);

      if (m_LastHit.m_Flags & Ark::Collision::ENTITY)
      {
	 ypos -= ui->GetStringHeight (fp,m_LastHit.m_Entity->m_Name);
	 ui->DrawString (fp, m_LastHit.m_Entity->m_Name, 5, ypos);
      }

      if (m_Client->GetPlayer())
	 m_MessageView->Render (ui, m_Client->GetPlayer());	 
   }

   void
   TerrainView::HandleMButton (bool down, int b, int x, int y)
   {
      if (m_MessageView->HandleMButton(down, b, x, y))
	 return;

      if (down && b == 1 && m_LastMousePos  != Ark::Vector3 (0.0, 0.0, 0.0))
      {
	 if (m_LastHit.m_Flags & Ark::Collision::WORLD)
	 {
	    m_Client->GetPlayer()->SetGoal(m_LastMousePos);
	    
	    // FIXME: hack !!!
	    m_Client->GetPlayer()->m_MState.Play
	       (Ark::ModelState::LOOP, "walk");
	 }
	 else if (m_LastHit.m_Flags & Ark::Collision::ENTITY)
	 {
	    m_Client->GetPlayer()->AddMessage
		     (m_LastHit.m_Entity, "MOUSEHIT");

	 }
      }
   }

   void
   TerrainView::HandleKey (bool down, int mods, int key)
   {
      if (down)
      {
	 switch (key)
	 {
	    case SDLK_PAGEUP: m_RotXSpeed = 60.0; break;
	    case SDLK_PAGEDOWN: m_RotXSpeed = -60.0; break;
	    case SDLK_LEFT:  m_RotYSpeed = -100.0; break;
	    case SDLK_RIGHT: m_RotYSpeed = 100.0; break;
	    case SDLK_UP:    m_ScaleSpeed = -1.0; break;
	    case SDLK_DOWN:  m_ScaleSpeed = 1.0; break;
	    case SDLK_SPACE:
	       m_Client->GetPlayer()->m_MState.Play
		  (Ark::ModelState::ONCE, "ref_shoot_crowbar");
	       break;

#if 0
	    case 'f':
	       Ark::g_UpdateVis = !Ark::g_UpdateVis;
	       break;

	    case 'e':
	       Ark::g_DrawEntities = !Ark::g_DrawEntities;
	       break;

	    case 't':
	       Ark::g_DrawTerrain = !Ark::g_DrawTerrain;
	       break;

	    case 'w':
	       Ark::g_WireFrame = !Ark::g_WireFrame;
	 	   break;
#endif
	    case SDLK_F11:
	       m_Client->GetCache()->Dump();
	    	    
	 }  
      }
      else
      {
	 switch (key)
	 {
	    case SDLK_PAGEUP:
	    case SDLK_PAGEDOWN:
	       m_RotXSpeed = 0.0;
	       break;

	    case SDLK_LEFT:
	    case SDLK_RIGHT:
	       m_RotYSpeed = 0.0;
	       break;

	    case SDLK_UP:
	    case SDLK_DOWN:
	       m_ScaleSpeed = 0.0;
	       break;
	 }
      }
   }

   void
   TerrainView::HandleMotion (int state, int x, int y)
   {}
}
