/* $Id: SkyDome.cpp,v 1.1 2002/08/28 22:44:53 mrq Exp $
**
** Ark - Libraries, Tools & Programs for MMORPG developpements.
** Copyright (C) 1999-2000 The Contributors of the Ark Project
** Please see the file "AUTHORS" for a list of contributors
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef WIN32
 #include <windows.h>
#endif

#include <GL/gl.h>

#include <Ark/ArkRenderer.h>
#include <Ark/ArkPrimBlock.h>
#include <Modules/HeightField/SkyDome.h>

namespace Ark
{

   Material*
   SkyBox::GetMaterial(const String &name)
   {
      String matname = m_Config.GetStr(name, String());

      if (matname.empty())
	 return 0;

      Material *mat;
      m_Cache->Get (V_MATERIAL, matname, &mat);

      return mat;
   }

   SkyBox::SkyBox (Cache *cache, Config& cfg)
       : m_Cache(cache), m_Config(cfg)
   {
      m_Materials[0] = GetMaterial ("skybox::North"); //FR
      m_Materials[1] = GetMaterial ("skybox::Top"); //TP
      m_Materials[3] = GetMaterial ("skybox::West"); //LE
      m_Materials[4] = GetMaterial ("skybox::East"); //RI
      m_Materials[2] = GetMaterial ("skybox::South"); //BT

      scalar w = cfg.GetScalar ("skybox::Width", 50.0);
      scalar h = cfg.GetScalar ("skybox::Height", 75.0);

      h=w=40.f;
	  
      m_VB.SetFormat (VertexBuffer::VB_HAS_UV0 |
		      VertexBuffer::VB_HAS_COORD |
		      VertexBuffer::VB_HAS_COLOR);
      m_VB.Resize (20);

      SetCoord (0,   -w, -h, -w,   0, 1);
      SetCoord (1,    w, -h, -w,   1, 1);
      SetCoord (2,    w,  h, -w,   1, 0);
      SetCoord (3,   -w,  h, -w,   0, 0);

      SetCoord (4,   -w,  h, -w,   0, 1);
      SetCoord (5,    w,  h, -w,   1, 1);
      SetCoord (6,    w,  h,  w,   1, 0);
      SetCoord (7,   -w,  h,  w,   0, 0);

      SetCoord (8,  -w, -h,   w,   0, 1);
      SetCoord (9,  -w, -h,  -w,   1, 1);
      SetCoord (10,  -w,  h, -w,   1, 0);
      SetCoord (11,  -w,  h,  w,   0, 0);

      SetCoord (12,   w, -h, -w,   0, 1);
      SetCoord (13,   w, -h,  w,   1, 1);
      SetCoord (14,   w,  h,  w,   1, 0);
      SetCoord (15,   w,  h, -w,   0, 0);

      SetCoord (16,   w, -h,  w,   0, 1);
      SetCoord (17,  -w, -h,  w,   1, 1);
      SetCoord (18,  -w,  h,  w,   1, 0);
      SetCoord (19,   w,  h,  w,   0, 0);
   }

   void
   SkyBox::SetCoord (int n,
		     scalar x, scalar y, scalar z,
		     scalar u, scalar v)
   {
      RGBA wh = {255,255,255,255};
      m_VB.Coord(n) = Vector3 (x, y, z);
      m_VB.UV0(n) = Vector2 (u, v);
      m_VB.Color4(n) = wh;
   }


   SkyBox::~SkyBox ()
   {
      for (int i = 0; i < 5; i++) 
      {
	  Material* mat = m_Materials[i];
	  
	  if (mat)
	      mat->Unref();
      }
   }

   void
   SkyBox::Render (Renderer &renderer, const Vector3 &position)
   {
      unsigned int i;

      //std::cerr << "Rendering skybox at " << position << "\n";
      
      for (i=0; i<20; ++i)
	 m_VB.Coord(i) += position;


      VertexBuffer vb;
      vb.SetFormat (VertexBuffer::VB_HAS_UV0 | VertexBuffer::VB_HAS_COORD);
      vb.Resize (4);

      vb.Coord(0);

      //renderer.RenderBlock (*m_Materials[0], PRIM_TRIANGLES, vb, 4);
      renderer.SetActiveVB (m_VB);
      renderer.LockVB (0, 20);

      uint idx = 0;
      for (i=0; i<5; ++i, idx+=4)
      {
	 if (m_Materials[i] == NULL)
	    continue;

	 ushort indices[4];
	 indices[0] = idx+2;
	 indices[1] = idx+1;
	 indices[2] = idx;
	 indices[3] = idx+3;
	 
	 //std::cerr << "quad " << i <<  m_Materials[i]->Dump (true) << "\n";
	 
	 renderer.RenderBlock (*m_Materials[i], PRIM_TRIANGLE_FAN,
			       indices, 4);
      }
   
      renderer.UnlockVB();

      for (i=0; i<20; ++i)
	 m_VB.Coord(i) -= position;

   }

}
